@@ -24,7 +24,7 @@
 #include "archive.h"
 
 gboolean is7zip_mhe(const gchar *);
-void xa_7zip_add(XArchive *, GSList *, gchar *);
+void xa_7zip_add(XArchive *, GSList *);
 void xa_7zip_ask(XArchive *);
 void xa_7zip_delete(XArchive *, GSList *);
 gboolean xa_7zip_extract(XArchive *, GSList *);
