@@ -33,15 +33,12 @@ static gboolean no_focus (GtkWidget *widget, GtkDirectionType direction, gpointe
 	return TRUE;
 }
 
-static void toggle_overwrite_update_freshen (GtkToggleButton *button, Add_dialog_data *data)
+static void toggle_update_freshen (GtkToggleButton *button, Add_dialog_data *data)
 {
 	gboolean active = gtk_toggle_button_get_active(button);
 
 	if (active)
 	{
-		if (button != GTK_TOGGLE_BUTTON(data->overwrite))
-			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data->overwrite), FALSE);
-
 		if (button != GTK_TOGGLE_BUTTON(data->update))
 			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data->update), FALSE);
 
@@ -163,24 +160,19 @@ Add_dialog_data *xa_create_add_dialog()
 	vbox3 = gtk_vbox_new (TRUE, 0);
 	gtk_container_add (GTK_CONTAINER (alignment4),vbox3);
 
-	add_dialog->overwrite = gtk_check_button_new_with_mnemonic(_("Overwrite existing files"));
-	gtk_button_set_focus_on_click(GTK_BUTTON(add_dialog->overwrite), FALSE);
-	gtk_box_pack_start(GTK_BOX(vbox3), add_dialog->overwrite, FALSE, FALSE, 0);
-	g_signal_connect(G_OBJECT(add_dialog->overwrite), "toggled", G_CALLBACK(toggle_overwrite_update_freshen), add_dialog);
-	g_signal_connect(G_OBJECT(add_dialog->overwrite), "focus", G_CALLBACK(no_focus), NULL);
-
 	add_dialog->update = gtk_check_button_new_with_mnemonic(_("Update existing files and add new ones"));
 	gtk_button_set_focus_on_click (GTK_BUTTON (add_dialog->update), FALSE);
 	gtk_widget_set_tooltip_text(add_dialog->update, _("This option will add any new files and update any files which are already in the archive but older there"));
 	gtk_box_pack_start (GTK_BOX (vbox3), add_dialog->update, FALSE, FALSE, 0);
-	g_signal_connect(G_OBJECT(add_dialog->update), "toggled", G_CALLBACK(toggle_overwrite_update_freshen), add_dialog);
+	g_signal_connect(G_OBJECT(add_dialog->update), "toggled", G_CALLBACK(toggle_update_freshen), add_dialog);
+	g_signal_connect(G_OBJECT(add_dialog->update), "focus", G_CALLBACK(no_focus), NULL);
 
 	add_dialog->freshen = gtk_check_button_new_with_mnemonic(_("Freshen existing files only"));
 
 	gtk_button_set_focus_on_click (GTK_BUTTON (add_dialog->freshen), FALSE);
 	gtk_widget_set_tooltip_text(add_dialog->freshen, _("This option will only add files which are already in the archive but older there; unlike the update option it will not add any new files"));
 	gtk_box_pack_start (GTK_BOX (vbox3), add_dialog->freshen, FALSE, FALSE, 0);
-	g_signal_connect(G_OBJECT(add_dialog->freshen), "toggled", G_CALLBACK(toggle_overwrite_update_freshen), add_dialog);
+	g_signal_connect(G_OBJECT(add_dialog->freshen), "toggled", G_CALLBACK(toggle_update_freshen), add_dialog);
 
 	add_dialog->recurse = gtk_check_button_new_with_mnemonic (_("Include subdirectories"));
 	gtk_button_set_focus_on_click (GTK_BUTTON (add_dialog->recurse), FALSE);
@@ -309,14 +301,10 @@ void xa_set_add_dialog_options(Add_dialog_data *add_dialog,XArchive *archive)
 	gtk_widget_set_sensitive(add_dialog->store_path, full_path);
 	gtk_widget_set_sensitive(add_dialog->update, archive->can_update[1]);
 	gtk_widget_set_sensitive(add_dialog->freshen, archive->can_freshen[1]);
-	gtk_widget_set_sensitive(add_dialog->recurse, TRUE);
+	gtk_widget_set_sensitive(add_dialog->recurse, !archive->can_recurse);
 	gtk_widget_set_sensitive(add_dialog->remove_files, archive->can_move);
 	gtk_widget_set_sensitive(add_dialog->solid_archive, archive->can_solid);
 
-	if ((!gtk_widget_is_sensitive(add_dialog->update) && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(add_dialog->update))) ||
-	    (!gtk_widget_is_sensitive(add_dialog->freshen) && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(add_dialog->freshen))))
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(add_dialog->overwrite), TRUE);
-
 	normal = (archive->compressor.least <= archive->compressor.best);
 
 	if (normal)
@@ -431,14 +419,14 @@ void xa_parse_add_dialog_options (XArchive *archive,Add_dialog_data *add_dialog)
 			if (!archive->do_full_path)
 				archive->child_dir = g_path_get_dirname(list->data);
 
-			xa_execute_add_commands(archive, list, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(add_dialog->recurse)));
+			xa_execute_add_commands(archive, list, gtk_widget_is_sensitive(add_dialog->recurse) && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(add_dialog->recurse)), gtk_widget_is_sensitive(add_dialog->recurse));
 		}
 	}
 	gtk_widget_destroy(add_dialog->compression_scale);
 	gtk_widget_hide (add_dialog->dialog1);
 }
 
-void xa_execute_add_commands (XArchive *archive, GSList *list, gboolean recurse)
+void xa_execute_add_commands (XArchive *archive, GSList *list, gboolean recurse, gboolean all)
 {
 	gchar *new_path = NULL;
 	gchar *esc2, *dest, *org_path = NULL;
@@ -525,7 +513,7 @@ void xa_execute_add_commands (XArchive *archive, GSList *list, gboolean recurse)
 
 	while (list)
 	{
-		xa_recurse_local_directory((gchar*) list->data, &dirlist, archive->do_full_path, recurse);
+		xa_recurse_local_directory((gchar*) list->data, &dirlist, archive->do_full_path, recurse, all);
 		list = list->next;
 	}
 	files = xa_collect_filenames(archive, dirlist);
