@@ -37,7 +37,6 @@ typedef struct Add_dialog_data
 	GtkWidget *no_store_path;
 	GtkWidget *option_notebook_vbox;
 	GtkWidget *remove_files;
-	GtkWidget *overwrite;
 	GtkWidget *update;
 	GtkWidget *freshen;
 	GtkWidget *solid_archive;
@@ -55,7 +54,7 @@ typedef struct Add_dialog_data
 } Add_dialog_data;
 
 Add_dialog_data *xa_create_add_dialog();
-void xa_execute_add_commands(XArchive *, GSList *, gboolean);
+void xa_execute_add_commands(XArchive *, GSList *, gboolean, gboolean);
 void xa_parse_add_dialog_options(XArchive *, Add_dialog_data *);
 void xa_set_add_dialog_options(Add_dialog_data *, XArchive *);
 
