@@ -145,7 +145,12 @@ static void xa_process_exit (GPid pid, gint status, XArchive *archive)
 static void xa_delete_working_directory (XArchive *archive)
 {
 	if (xa_main_window)
-		xa_launch_external_program("rm -rf", archive->working_dir);
+	{
+		gchar *working_dir = g_shell_quote(archive->working_dir);
+
+		xa_launch_external_program("rm -rf", working_dir);
+		g_free(working_dir);
+	}
 	else
 	{
 		char *argv[4];
@@ -533,6 +538,24 @@ gboolean xa_create_working_directory (XArchive *archive)
 	return TRUE;
 }
 
+gchar *xa_create_working_subdirectory (XArchive *archive)
+{
+	gchar *subdir;
+
+	if (!xa_create_working_directory(archive))
+		return NULL;
+
+	subdir = g_strconcat(archive->working_dir, "/xa-tmp.XXXXXX", NULL);
+
+	if (g_mkdtemp(subdir))
+		return subdir;
+	else
+	{
+		g_free(subdir);
+		return NULL;
+	}
+}
+
 gchar *xa_create_containing_directory (XArchive *archive, const gchar *path)
 {
 	gchar *stem, *dir;
