@@ -29,9 +29,11 @@ typedef enum
 	XARCHIVETYPE_AR,
 	XARCHIVETYPE_ARJ,
 	XARCHIVETYPE_BZIP2,
+	XARCHIVETYPE_CAB,
 	XARCHIVETYPE_COMPRESS,
 	XARCHIVETYPE_CPIO,
 	XARCHIVETYPE_GZIP,
+	XARCHIVETYPE_ISO,
 	XARCHIVETYPE_LHA,
 	XARCHIVETYPE_LRZIP,
 	XARCHIVETYPE_LZ4,
@@ -40,6 +42,7 @@ typedef enum
 	XARCHIVETYPE_LZOP,
 	XARCHIVETYPE_RAR,
 	XARCHIVETYPE_RPM,
+	XARCHIVETYPE_SQUASHFS,
 	XARCHIVETYPE_TAR,
 	XARCHIVETYPE_TAR_BZIP2,
 	XARCHIVETYPE_TAR_COMPRESS,
@@ -53,6 +56,7 @@ typedef enum
 	XARCHIVETYPE_TAR_ZSTD,
 	XARCHIVETYPE_XZ,
 	XARCHIVETYPE_ZIP,
+	XARCHIVETYPE_ZPAQ,
 	XARCHIVETYPE_ZSTD,
 	XARCHIVETYPE_TYPES,
 	XARCHIVETYPE_UNKNOWN,
@@ -186,6 +190,7 @@ struct _XArchive
 	gboolean can_freshen[2];     // can limit activity to only changed files
 	gboolean can_move;           // can delete files after adding
 	gboolean can_solid;          // can create a solid archive
+	gboolean can_recurse;        // can automatically recursion through subdirectories
 	gboolean can_compress;       // can compress at different levels
 	/* instructions */
 	gboolean do_full_path;   // extract_func, (xa_execute_add_commands)
@@ -239,6 +244,7 @@ gchar *xa_build_full_path_name_from_entry(XEntry *);
 void xa_clean_archive_structure(XArchive *);
 gchar *xa_create_containing_directory(XArchive *, const gchar *);
 gboolean xa_create_working_directory(XArchive *);
+gchar *xa_create_working_subdirectory(XArchive *);
 void xa_detect_encrypted_archive(XArchive *);
 void xa_fill_dir_sidebar(XArchive *, gboolean);
 void xa_fill_list_with_recursed_entries(XEntry *, GSList **);
