@@ -263,31 +263,22 @@ gboolean xa_arj_extract (XArchive *archive, GSList *file_list)
 	{
 		if (xa_create_working_directory(archive))
 		{
-			gchar *files_str, *extraction_dir, *archive_path, *move;
-
-			files_str = xa_escape_bad_chars(files->str, "\"");
-			extraction_dir = xa_quote_shell_command(archive->extraction_dir, FALSE);
-			archive_path = xa_quote_shell_command(archive->path[0], TRUE);
-
-			if (strcmp(archive->extraction_dir, archive->working_dir) == 0)
-				move = g_strdup("");
-			else
-				move = g_strconcat(" mv",
-				                   archive->do_overwrite ? " -f" : (archive->do_update ? " -fu" : " -n"),
-				                   " --", *files->str ? files_str : " `ls -A`", " ",
-				                   extraction_dir, NULL);
+			GString *dir_contents;
 
 			archive->child_dir = g_strdup(archive->working_dir);
+			dir_contents = xa_quote_dir_contents(archive->child_dir);
 
-			command = g_strdup("sh -c \"exec rm -f -- `ls -A`\"");
+			command = g_strconcat("rm -f --", dir_contents->str, NULL);
 			result = xa_run_command(archive, command);
 
 			g_free(command);
 			command = NULL;
 
+			g_string_free(dir_contents, TRUE);
+
 			if (result)
 			{
-				command = g_strconcat(archiver[archive->type].program[0], " e ", archive_path, NULL);
+				command = g_strconcat(archiver[archive->type].program[0], " e ", archive->path[1], NULL);
 				archive->status = XARCHIVESTATUS_EXTRACT;   // restore status
 				result = xa_run_command(archive, command);
 
@@ -296,15 +287,20 @@ gboolean xa_arj_extract (XArchive *archive, GSList *file_list)
 
 				if (result)
 				{
-					command = g_strconcat("sh -c \"exec", move, "\"", NULL);
-					archive->status = XARCHIVESTATUS_EXTRACT;   // restore status
+					if (strcmp(archive->extraction_dir, archive->working_dir) != 0)
+					{
+						dir_contents = xa_quote_dir_contents(archive->child_dir);
+
+						command = g_strconcat("mv",
+						                      archive->do_overwrite ? " -f" : (archive->do_update ? " -fu" : " -n"),
+						                      " --", *files->str ? files->str : dir_contents->str, " ",
+						                      archive->extraction_dir, NULL);
+						archive->status = XARCHIVESTATUS_EXTRACT;   // restore status
+
+						g_string_free(dir_contents, TRUE);
+					}
 				}
 			}
-
-			g_free(move);
-			g_free(archive_path);
-			g_free(extraction_dir);
-			g_free(files_str);
 		}
 		else
 			command = g_strdup("sh -c \"\"");
