@@ -25,9 +25,9 @@
 
 typedef struct Extract_dialog_data
 {
-	GtkWidget *dialog1,*dialog_vbox1,*destination_path_entry,*ensure_directory,*overwrite_check,*extract_full,*touch,*fresh,*update,*all_files_radio,*files_radio,
-	 *selected_radio,*entry2,*password_entry;
+	GtkWidget *dialog1, *dialog_vbox1, *destination_path_entry, *ensure_directory, *overwrite_check, *touch, *fresh, *update, *all_files_radio, *files_radio, *selected_radio, *entry2, *full_paths_radio, *relative_paths_radio, *no_paths_radio, *password_entry;
 	gchar *string;
+	XArchive *archive;
 } Extract_dialog_data;
 
 typedef struct Multi_extract_data
@@ -42,6 +42,7 @@ typedef struct Multi_extract_data
 
 Extract_dialog_data *xa_create_extract_dialog();
 Multi_extract_data *xa_create_multi_extract_dialog();
+void xa_execute_extract_commands(XArchive *, GSList *, gboolean);
 void xa_multi_extract_dialog(Multi_extract_data *);
 void xa_multi_extract_dialog_add_file(gchar *, Multi_extract_data *);
 void xa_parse_extract_dialog_options(XArchive * , Extract_dialog_data *, GtkTreeSelection *);
