@@ -50,25 +50,25 @@
 
 static gpointer item[7];
 static gchar *filename;
-static gboolean data_line, last_line;
+static gboolean first_line, data_line, last_line;
 static gboolean lrzip_can_password;
 static gboolean zstd_can_list, zstd_can_test;
 
 void xa_gzip_et_al_check_lrzip (const gchar *path)
 {
-	gchar *command, *output;
+	gchar *command, *output = NULL;
 
 	command = g_strconcat(path, " -h", NULL);
 	g_spawn_command_line_sync(command, NULL, &output, NULL, NULL);
 	g_free(command);
 
-	lrzip_can_password = (strstr(output, "-e, --encrypt[=pass") != NULL);
+	lrzip_can_password = (output && strstr(output, "-e, --encrypt[=pass"));
 	g_free(output);
 }
 
 gchar *xa_gzip_et_al_check_zstd (const gchar *compressor, const gchar *decompressor, gboolean *is_compressor)
 {
-	gchar *path, *command, *output;
+	gchar *path, *command, *output = NULL;
 	gboolean found_compressor = FALSE;
 
 	path = g_find_program_in_path(compressor);
@@ -86,7 +86,7 @@ gchar *xa_gzip_et_al_check_zstd (const gchar *compressor, const gchar *decompres
 	g_free(command);
 
 	/* check whether decompression is available */
-	if (strstr(output, "\n -d "))
+	if (output && strstr(output, "\n -d "))
 	{
 		if (found_compressor)
 			*is_compressor = (strstr(output, "\n -# ") != NULL);
@@ -129,6 +129,54 @@ gchar *xa_gzip_et_al_get_command (const gchar *program, gchar *workfile, gchar *
 	return command;
 }
 
+static compressor_t xa_gzip_et_al_compressor (XArchive *archive)
+{
+	compressor_t bzip2_compressor = {FALSE, 1, 9, 9, 1};
+	compressor_t compress_compressor = {FALSE, 9, 16, 16, 1};
+	compressor_t gzip_compressor = {FALSE, 1, 6, 9, 1};
+	compressor_t lrzip_compressor = {TRUE, 1, 7, 9, 1};
+	compressor_t lz4_compressor = {FALSE, 1, 1, 9, 1};
+	compressor_t lzma_compressor = {FALSE, 1, 7, 9, 1};
+	compressor_t lzop_compressor = {FALSE, 1, 3, 9, 1};
+	compressor_t xz_compressor = {FALSE, 0, 6, 9, 1};
+	compressor_t zstd_compressor = {FALSE, 1, 3, 19, 1};
+	compressor_t void_compressor = {FALSE, 0, 0, 0, 0};
+
+	switch (archive->type)
+	{
+		case XARCHIVETYPE_BZIP2:
+			return bzip2_compressor;
+
+		case XARCHIVETYPE_COMPRESS:
+			return compress_compressor;
+
+		case XARCHIVETYPE_GZIP:
+			return gzip_compressor;
+
+		case XARCHIVETYPE_LZIP:
+		case XARCHIVETYPE_XZ:
+			return xz_compressor;
+
+		case XARCHIVETYPE_LRZIP:
+			return lrzip_compressor;
+
+		case XARCHIVETYPE_LZ4:
+			return lz4_compressor;
+
+		case XARCHIVETYPE_LZMA:
+			return lzma_compressor;
+
+		case XARCHIVETYPE_LZOP:
+			return lzop_compressor;
+
+		case XARCHIVETYPE_ZSTD:
+			return zstd_compressor;
+
+		default:
+			return void_compressor;
+	}
+}
+
 static void xa_gzip_et_al_can (XArchive *archive, gboolean can)
 {
 	archive->can_test = (can && !compress && (!zstd || zstd_can_test));
@@ -139,6 +187,10 @@ static void xa_gzip_et_al_can (XArchive *archive, gboolean can)
 
 	/* only if archive is new and empty */
 	archive->can_add = (can && archiver[archive->type].is_compressor);
+
+	archive->can_compress = (can && archiver[archive->type].is_compressor);
+	archive->compressor = xa_gzip_et_al_compressor(archive);
+	archive->compression = archive->compressor.preset;
 }
 
 void xa_gzip_et_al_ask (XArchive *archive)
@@ -163,7 +215,7 @@ static void xa_gzip_et_al_parse_output (gchar *line, XArchive *archive)
 	else if (archive->type == XARCHIVETYPE_LZIP)
 	{
 		/* heading? */
-		if (line[3] == 'u')
+		if (*g_strchug(line) == 'u')
 			return;
 		else
 		{
@@ -272,26 +324,50 @@ static void xa_gzip_et_al_parse_lrzip (gchar *line, XArchive *archive)
 
 	if (!data_line)
 	{
-		LAST_ITEM(filename);
+		if (first_line)
+		{
+			first_line = FALSE;
 
-		filename[strlen(filename) - 1] = 0;   // remove colon
+			/* pre-v0.650 */
+			if (*line != '\n')
+			{
+				gchar *colon = g_strrstr(line, ":\n");
 
-		if (g_str_has_suffix(filename, ".lrz"))
-			*(line - 6) = 0;
+				if (colon) *colon = '\n';   // remove colon
 
-		filename = g_path_get_basename(filename);
-		data_line = TRUE;
+				goto filename;
+			}
+		}
+
+		/* since v0.650 */
+		IF_ITEM_LINE("File:")
+		{
+filename:
+			LAST_ITEM(filename);
+
+			if (g_str_has_suffix(filename, ".lrz"))
+				*(line - 5) = 0;
+
+			filename = g_path_get_basename(filename);
+			data_line = TRUE;
+		}
 
 		return;
 	}
 
+	line = g_strchug(line);
+
 	IF_ITEM_LINE("lrzip version:")
 	{
 		NEXT_ITEM(item[6]);
 		item[6] = g_strdup(item[6]);
 	}
+	/* pre-v0.650 */
 	else IF_ITEM_LINE("Compression:")
 		DUPE_ITEM(item[5]);
+	/* since v0.650 */
+	else IF_ITEM_LINE("Compression Method:")
+		DUPE_ITEM(item[5]);
 	else IF_ITEM_LINE("Decompressed file size:")
 		DUPE_ITEM(item[0]);
 	else IF_ITEM_LINE("Compressed file size:")
@@ -299,6 +375,13 @@ static void xa_gzip_et_al_parse_lrzip (gchar *line, XArchive *archive)
 	else IF_ITEM_LINE("Compression ratio:")
 	{
 		NEXT_ITEM(item[2]);
+
+		/* since v0.650 */
+		if (g_str_has_suffix(item[2], "x"))
+			*((char *) item[2] + strlen(item[2]) - 1) = 0;
+		else if (*(char *) item[2] == 'U')                 // Unavailable
+			item[2] = "1";
+
 		item[2] = g_strconcat(item[2], ":1", NULL);
 		last_line = TRUE;
 	}
@@ -328,7 +411,8 @@ static void xa_gzip_et_al_parse_lrzip (gchar *line, XArchive *archive)
 
 static void xa_gzip_et_al_parse_zstd (gchar *line, XArchive *archive)
 {
-	static gchar *zstandard;
+	static gchar *zstdframes;
+	gchar *skipframes;
 	XEntry *entry;
 	char *pos;
 
@@ -337,29 +421,39 @@ static void xa_gzip_et_al_parse_zstd (gchar *line, XArchive *archive)
 	if (last_line)
 		return;
 
+	if (first_line)
+		/* post-v1.3.2? (no header any more) */
+		data_line = (strncmp(line, "===========================================", 43) != 0);
+
 	if (!data_line)
 	{
+		first_line = FALSE;
+		/* pre-v1.3.2? (end of header) */
 		data_line = (strncmp(line, "Number of files listed:", 23) == 0);
 		return;
 	}
 
-	if ((pos = strstr(line, " (1/1):\n")))
+	if ((pos = strstr(line, first_line ? "\n" : "(1/1):\n")))
 	{
 		*pos = 0;
+
 		LAST_ITEM(filename);
 
-		if (g_str_has_suffix(filename, ".zst"))
-			*(line - 4) = 0;
+		if (g_str_has_suffix(filename, ".zst "))
+			*(line - 5) = 0;
 
 		filename = g_path_get_basename(filename);
 	}
 	else IF_ITEM_LINE("# Zstandard Frames:")
-		DUPE_ITEM(zstandard);
+	{
+		DUPE_ITEM(zstdframes);
+		item[6] = g_strconcat(zstdframes, "/0", NULL);
+	}
 	else IF_ITEM_LINE("# Skippable Frames:")
 	{
-		NEXT_ITEM(item[6]);
-		item[6] = g_strconcat(zstandard, "/", item[6], NULL);
-		g_free(zstandard);
+		NEXT_ITEM(skipframes);
+		g_free(item[6]);
+		item[6] = g_strconcat(zstdframes, "/", skipframes, NULL);
 	}
 	else IF_ITEM_LINE("Compressed Size:")
 	{
@@ -384,9 +478,14 @@ static void xa_gzip_et_al_parse_zstd (gchar *line, XArchive *archive)
 		last_line = TRUE;
 	}
 
+	first_line = FALSE;
+
 	if (!last_line)
 		return;
 
+	if (!item[0])
+		item[0] = g_strdup("0");
+
 	entry = xa_set_archive_entries_for_each_row(archive, filename, item);
 
 	if (entry)
@@ -403,6 +502,7 @@ static void xa_gzip_et_al_parse_zstd (gchar *line, XArchive *archive)
 	g_free(item[5]);
 	g_free(item[6]);
 	g_free(filename);
+	g_free(zstdframes);
 }
 
 static void xa_gzip_et_al_globally_stored_entry (gchar *line, XArchive *archive)
@@ -481,84 +581,86 @@ void xa_gzip_et_al_list (XArchive *archive)
 
 		file = fopen(archive->path[0], "r");
 
-		if (file)
-		{
-			workfile = g_strconcat(archive->working_dir, "/", framefile, NULL);
-			wfile = fopen(workfile, "w");
-
-			if (wfile)
-			{
-				skip = (archive->tag == 'm' ? 8 : 16);   // magic
-
-				/* original (uncompressed) data size */
-				fseek(file, skip, SEEK_SET);
-				fread(&uncompressed, sizeof(uncompressed), 1, file);
-
-				/* create a lz4 frame header */
-
-				fwrite(LZ4_MAGIC, 4, 1, wfile);
-
-				flg = (0 << 7) + (1 << 6)   // version number
-				               + (1 << 5)   // block independence flag
-				               + (0 << 4)   // block checksum flag
-				               + (0 << 3)   // content size flag
-				               + (0 << 2)   // content checksum flag
-				               + (0 << 1)   // reserved
-				               + (0 << 0);  // dictionary id flag
-				fwrite(&flg, 1, 1, wfile);
-
-				if (le32toh(uncompressed) < 64 * 1024)          // 64 KB
-				{
-					bd = 0x40;
-					hc = 0x82;
-				}
-				else if (le32toh(uncompressed) < 256 * 1024)    // 256 KB
-				{
-					bd = 0x50;
-					hc = 0xfb;
-				}
-				else if (le32toh(uncompressed) < 1024 * 1024)   // 1 MB
-				{
-					bd = 0x60;
-					hc = 0x51;
-				}
-				else                                            // 4 MB
-				{
-					bd = 0x70;
-					hc = 0x73;
-				}
-
-				fwrite(&bd, 1, 1, wfile);   // block maximum size
-				fwrite(&hc, 1, 1, wfile);   // header checksum
-
-				offset = skip + 4;
-
-				fseek(file, 0, SEEK_END);
-				blocksize = htole32((uint32_t) (ftell(file) - offset));
-				fwrite(&blocksize, sizeof(blocksize), 1, wfile);
-
-				fseek(file, offset, SEEK_SET);
-
-				/* copy lz4 data */
-				while ((bytes = fread(buffer, 1, sizeof(buffer), file)) > 0)
-					fwrite(buffer, 1, bytes, wfile);
-
-				fwrite(endmark, sizeof(endmark), 1, wfile);
-
-				fclose(wfile);
-			}
-			else
-			{
-				fclose(file);
-				g_free(workfile);
-				return;
-			}
-
-			fclose(file);
-		}
-		else
+		if (!file)
 			return;
 
+		workfile = g_strconcat(archive->working_dir, "/", framefile, NULL);
+		wfile = fopen(workfile, "w");
+
+		if (!wfile)
+		{
+			fclose(file);
+			g_free(workfile);
+			return;
+		}
+
+		skip = (archive->tag == 'm' ? 8 : 16);   // magic
+
+		/* original (uncompressed) data size */
+		fseek(file, skip, SEEK_SET);
+
+		if (fread(&uncompressed, sizeof(uncompressed), 1, file) != 1)
+		{
+			fclose(file);
+			fclose(wfile);
+			g_free(workfile);
+			return;
+		}
+
+		/* create a lz4 frame header */
+
+		fwrite(LZ4_MAGIC, 4, 1, wfile);
+
+		flg = (0 << 7) + (1 << 6)   // version number
+		               + (1 << 5)   // block independence flag
+		               + (0 << 4)   // block checksum flag
+		               + (0 << 3)   // content size flag
+		               + (0 << 2)   // content checksum flag
+		               + (0 << 1)   // reserved
+		               + (0 << 0);  // dictionary id flag
+		fwrite(&flg, 1, 1, wfile);
+
+		if (le32toh(uncompressed) < 64 * 1024)          // 64 KB
+		{
+			bd = 0x40;
+			hc = 0x82;
+		}
+		else if (le32toh(uncompressed) < 256 * 1024)    // 256 KB
+		{
+			bd = 0x50;
+			hc = 0xfb;
+		}
+		else if (le32toh(uncompressed) < 1024 * 1024)   // 1 MB
+		{
+			bd = 0x60;
+			hc = 0x51;
+		}
+		else                                            // 4 MB
+		{
+			bd = 0x70;
+			hc = 0x73;
+		}
+
+		fwrite(&bd, 1, 1, wfile);   // block maximum size
+		fwrite(&hc, 1, 1, wfile);   // header checksum
+
+		offset = skip + 4;
+
+		fseek(file, 0, SEEK_END);
+		blocksize = htole32((uint32_t) (ftell(file) - offset));
+		fwrite(&blocksize, sizeof(blocksize), 1, wfile);
+
+		fseek(file, offset, SEEK_SET);
+
+		/* copy lz4 data */
+		while ((bytes = fread(buffer, 1, sizeof(buffer), file)) > 0)
+			fwrite(buffer, 1, bytes, wfile);
+
+		fwrite(endmark, sizeof(endmark), 1, wfile);
+
+		fclose(wfile);
+		fclose(file);
+
 		archive->path[2] = g_strdup(workfile);
 		archive->path[3] = xa_escape_bad_chars(workfile, ESCAPES);
 		g_free(workfile);
@@ -652,6 +754,7 @@ void xa_gzip_et_al_list (XArchive *archive)
 					/* no further information will be available then */
 					break;
 
+				first_line = TRUE;
 				data_line = FALSE;
 				last_line = FALSE;
 
@@ -693,6 +796,7 @@ void xa_gzip_et_al_list (XArchive *archive)
 				item[0] = NULL;
 				item[2] = NULL;
 
+				first_line = TRUE;
 				data_line = FALSE;
 				last_line = FALSE;
 
@@ -832,69 +936,52 @@ gboolean xa_gzip_et_al_extract (XArchive *archive, GSList *file_list)
 	return result;
 }
 
-void xa_gzip_et_al_add (XArchive *archive, GSList *file_list, gchar *compression)
+void xa_gzip_et_al_add (XArchive *archive, GSList *file_list)
 {
 	GString *files;
-	gchar *move, *files_str, *archive_path, *password_str, *command;
+	gchar *compression, *files_str, *archive_path, *move, *password_str, *out, *command;
+	gboolean success;
 
-	if (archive->location_path != NULL)
-		archive->child_dir = g_strdup(archive->working_dir);
-
-	if (!compression)
-	{
-		switch (archive->type)
-		{
-			case XARCHIVETYPE_BZIP2:
-				compression = "9";
-				break;
-
-			case XARCHIVETYPE_COMPRESS:
-				compression = "16";
-				break;
-
-			case XARCHIVETYPE_GZIP:
-			case XARCHIVETYPE_LZIP:
-			case XARCHIVETYPE_XZ:
-				compression = "6";
-				break;
-
-			case XARCHIVETYPE_LZ4:
-				compression = "1";
-				break;
-
-			case XARCHIVETYPE_LRZIP:
-			case XARCHIVETYPE_LZMA:
-				compression = "7";
-				break;
-
-			case XARCHIVETYPE_LZOP:
-			case XARCHIVETYPE_ZSTD:
-				compression = "3";
-				break;
-
-			default:
-				break;
-		}
-	}
+	compression = g_strdup_printf("%hu", archive->compression);
 
 	files = xa_quote_filenames(file_list, NULL, TRUE);
 	files_str = xa_escape_bad_chars(files->str, "\"");
 	archive_path = xa_quote_shell_command(archive->path[2] ? archive->path[2] : archive->path[0], TRUE);
 
 	if (archive->do_move)
-		move = g_strconcat(" && rm", files_str, NULL);
+		move = g_strconcat(" rm --", files_str, NULL);
 	else
 		move = g_strdup("");
 
 	password_str = xa_gzip_et_al_password_str(archive->password, archive->type);
-	command = g_strconcat("sh -c \"exec ", archiver[archive->type].program[0], " -", compress ? "b " : (lrzip ? "L " : ""), compression, password_str, files_str, lrzip ? " -fo " : " -c > ", archive_path, move, "\"", NULL);
-	g_free(password_str);
 
+	if (lrzip)
+		out = g_strconcat(" -fo ", archive_path, " --", files_str, NULL);
+	else
+		out = g_strconcat(" -c --", files_str, " > ", archive_path, NULL);
+
+	command = g_strconcat("sh -c \"exec ", archiver[archive->type].program[0], " -", compress ? "b " : (lrzip ? (*compression == '0' ? "n" : "L ") : ""), lrzip && (*compression == '0') ? "" : compression, password_str, out, "\"", NULL);
+	success = xa_run_command(archive, command);
+
+	g_free(command);
+	command = NULL;
+
+	if (success)
+	{
+		command = g_strconcat("sh -c \"exec", move, "\"", NULL);
+		archive->status = XARCHIVESTATUS_ADD;   // restore status
+	}
+
+	g_free(out);
+	g_free(password_str);
 	g_free(move);
 	g_free(archive_path);
 	g_free(files_str);
 	g_string_free(files, TRUE);
+	g_free(compression);
+
+	if (command)
+		xa_run_command(archive, command);
 
-	xa_run_command(archive, command);
 	g_free(command);
 }
