@@ -31,11 +31,12 @@
                                          archive->type == XARCHIVETYPE_LZIP     || \
                                          archive->type == XARCHIVETYPE_LZMA     || \
                                          archive->type == XARCHIVETYPE_LZOP     || \
-                                         archive->type == XARCHIVETYPE_XZ)
+                                         archive->type == XARCHIVETYPE_XZ       || \
+                                         archive->type == XARCHIVETYPE_ZSTD)
 
 #define LZ4_MAGIC "\x04\x22\x4d\x18"
 
-void xa_gzip_et_al_add(XArchive *, GSList *, gchar *);
+void xa_gzip_et_al_add(XArchive *, GSList *);
 void xa_gzip_et_al_ask(XArchive *);
 void xa_gzip_et_al_check_lrzip(const gchar *);
 gchar *xa_gzip_et_al_check_zstd(const gchar *, const gchar *, gboolean *);
