@@ -317,7 +317,7 @@ failed:
 	archive[idx]->do_move = FALSE;
 	archive[idx]->do_solid = FALSE;
 
-	xa_execute_add_commands(archive[idx], list, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->allow_sub_dir)));
+	xa_execute_add_commands(archive[idx], list, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->allow_sub_dir)), TRUE);
 
 	g_string_free(pathname, TRUE);
 	g_slist_free_full(list, g_free);
