@@ -30,12 +30,11 @@ static gboolean data_line, last_line;
 
 gboolean xa_lha_check_program (gchar *path)
 {
-	gchar *output, *discard;
+	gchar *output = NULL;
 	gboolean full_lha;
 
-	g_spawn_command_line_sync(path, &output, &discard, NULL, NULL);
-	full_lha = (g_ascii_strncasecmp("Lhasa ", output, 6) != 0);
-	g_free(discard);
+	g_spawn_command_line_sync(path, &output, NULL, NULL, NULL);
+	full_lha = (output && (g_ascii_strncasecmp("Lhasa ", output, 6) != 0));
 	g_free(output);
 
 	return full_lha;
@@ -43,6 +42,8 @@ gboolean xa_lha_check_program (gchar *path)
 
 void xa_lha_ask (XArchive *archive)
 {
+	compressor_t lha_compressor = {FALSE, 5, 5, 7, 1};
+
 	archive->can_test = TRUE;
 	archive->can_extract = TRUE;
 	archive->can_add = archiver[archive->type].is_compressor;
@@ -52,6 +53,9 @@ void xa_lha_ask (XArchive *archive)
 	archive->can_overwrite = TRUE;
 	archive->can_update[1] = archiver[archive->type].is_compressor;
 	archive->can_move = archiver[archive->type].is_compressor;
+	archive->can_compress = archiver[archive->type].is_compressor;
+	archive->compressor = lha_compressor;
+	archive->compression = archive->compressor.preset;
 }
 
 static void xa_lha_parse_output (gchar *line, XArchive *archive)
@@ -212,16 +216,12 @@ gboolean xa_lha_extract (XArchive *archive, GSList *file_list)
 	return result;
 }
 
-void xa_lha_add (XArchive *archive, GSList *file_list, gchar *compression)
+void xa_lha_add (XArchive *archive, GSList *file_list)
 {
 	GString *files;
-	gchar *command;
+	gchar *compression, *command;
 
-	if (archive->location_path != NULL)
-		archive->child_dir = g_strdup(archive->working_dir);
-
-	if (!compression)
-		compression = "5";
+	compression = g_strdup_printf("%hu", archive->compression);
 
 	files = xa_quote_filenames(file_list, NULL, TRUE);
 	command = g_strconcat(archiver[archive->type].program[0],
@@ -230,6 +230,7 @@ void xa_lha_add (XArchive *archive, GSList *file_list, gchar *compression)
 	                      "o", compression, " ",
 	                      archive->path[1], files->str, NULL);
 	g_string_free(files,TRUE);
+	g_free(compression);
 
 	xa_run_command(archive, command);
 	g_free(command);
