@@ -33,12 +33,14 @@
 #include "pref_dialog.h"
 #include "rar.h"
 #include "rpm.h"
+#include "squashfs.h"
 #include "string_utils.h"
 #include "support.h"
 #include "tar.h"
 #include "unar.h"
 #include "window.h"
 #include "zip.h"
+#include "zpaq.h"
 
 #ifdef HAVE_SOCKET
 #include "socket.h"
@@ -103,7 +105,10 @@ static void xa_check_available_archivers ()
 
 	/* (un)compressors that can handle various types */
 
-	sevenz = g_find_program_in_path("7z");
+	sevenz = g_find_program_in_path("7zz");
+
+	if (!sevenz)
+		sevenz = g_find_program_in_path("7z");
 
 	if (!sevenz)
 	{
@@ -307,6 +312,23 @@ static void xa_check_available_archivers ()
 		archiver[type].add = FUNC(standard, xa_gzip_et_al_add, is7za, xa_7zip_add, lsar, NULL);
 	}
 
+	/* cab */
+
+	type = XARCHIVETYPE_CAB;
+	path = (is7z || is7za ? g_strconcat(sevenz, " -tcab", NULL) : NULL);
+
+	if (path)
+	{
+		archiver[type].program[0] = path;
+		archiver[type].type = g_slist_append(archiver[type].type, "cab");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.cab");
+
+		archiver[type].ask = xa_7zip_ask;
+		archiver[type].list = xa_7zip_list;
+		archiver[type].test = xa_7zip_test;
+		archiver[type].extract = xa_7zip_extract;
+	}
+
 	/* compress */
 
 	type = XARCHIVETYPE_COMPRESS;
@@ -440,6 +462,23 @@ static void xa_check_available_archivers ()
 		archiver[type].add = FUNC(standard, xa_gzip_et_al_add, is7za, xa_7zip_add, lsar, NULL);
 	}
 
+	/* ISO */
+
+	type = XARCHIVETYPE_ISO;
+	path = (is7z ? g_strconcat(sevenz, " -tiso", NULL) : NULL);
+
+	if (path)
+	{
+		archiver[type].program[0] = path;
+		archiver[type].type = g_slist_append(archiver[type].type, "iso");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.iso");
+
+		archiver[type].ask = xa_7zip_ask;
+		archiver[type].list = xa_7zip_list;
+		archiver[type].test = xa_7zip_test;
+		archiver[type].extract = xa_7zip_extract;
+	}
+
 	/* LHA */
 
 	type = XARCHIVETYPE_LHA;
@@ -728,6 +767,50 @@ static void xa_check_available_archivers ()
 		archiver[type].extract = FUNC(standard, xa_rpm_extract, is7z, xa_7zip_extract, unar, xa_unar_extract);
 	}
 
+	/* SquashFS */
+
+	type = XARCHIVETYPE_SQUASHFS;
+	path = g_find_program_in_path("unsquashfs");
+
+	standard = (path != NULL);
+
+	if (!standard)
+	{
+		if (is7z)
+			/* alternative uncompressor */
+			path = g_strconcat(sevenz, " -tsquashfs", NULL);
+	}
+	else
+	{
+		gchar *mksquashfs = g_find_program_in_path("mksquashfs");
+
+		if (mksquashfs)
+		{
+			archiver[type].program[1] = mksquashfs;
+			archiver[type].is_compressor = TRUE;
+		}
+	}
+
+	if (path)
+	{
+		archiver[type].program[0] = path;
+		archiver[type].type = g_slist_append(archiver[type].type, "squashfs");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.squashfs");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.sfs");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.sqfs");
+
+		/* snap package */
+		archiver[type].type = g_slist_append(archiver[type].type, "snap");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.snap");
+		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('s'));
+		archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
+
+		archiver[type].ask = (standard ? xa_squashfs_ask : xa_7zip_ask);
+		archiver[type].list = (standard ? xa_squashfs_list : xa_7zip_list);
+		archiver[type].extract = (standard ? xa_squashfs_extract : xa_7zip_extract);
+		archiver[type].add = (standard ? xa_squashfs_add : NULL);
+	}
+
 	/* tape archive */
 
 	type = XARCHIVETYPE_TAR;
@@ -907,6 +990,27 @@ static void xa_check_available_archivers ()
 		archiver[type].delete = FUNC(standard, xa_zip_delete, is7za, xa_7zip_delete, lsar, NULL);
 	}
 
+	/* zpaq */
+
+	type = XARCHIVETYPE_ZPAQ;
+	path = g_find_program_in_path("zpaq");
+
+	if (path)
+	{
+		archiver[type].is_compressor = TRUE;
+
+		archiver[type].program[0] = path;
+		archiver[type].type = g_slist_append(archiver[type].type, "zpaq");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.zpaq");
+
+		archiver[type].ask = xa_zpaq_ask;
+		archiver[type].list = xa_zpaq_list;
+		archiver[type].test = xa_zpaq_test;
+		archiver[type].extract = xa_zpaq_extract;
+		archiver[type].add = xa_zpaq_add;
+		archiver[type].delete = xa_zpaq_delete;
+	}
+
 	/* Zstandard */
 
 	type = XARCHIVETYPE_ZSTD;
@@ -1266,7 +1370,7 @@ int main (int argc, char **argv)
 				archive->do_move = FALSE;
 				archive->do_solid = FALSE;
 
-				xa_execute_add_commands(archive, files, TRUE);
+				xa_execute_add_commands(archive, files, TRUE, TRUE);
 			}
 		}
 		/* Switch -a */
