@@ -19,6 +19,7 @@
 
 #include <string.h>
 #include <unistd.h>
+#include <glib/gstdio.h>
 #include "pref_dialog.h"
 #include "add_dialog.h"
 #include "archive.h"
@@ -525,25 +526,39 @@ void xa_prefs_load_options(Prefs_dialog_data *prefs_data)
 	gint *add_coords = NULL;
 	gchar *value;
 	gsize coords_len = 0;
-	gchar *config_dir = NULL;
-	gchar *xarchiver_config_dir = NULL;
+	const gchar *config_dir;
+	gchar *xarchiver_config_dir, *old_dir, *old_config;
 	GKeyFile *xa_key_file = g_key_file_new();
 	GError *error = NULL;
 	gboolean unzip, toolbar;
 	gint idx, val;
 
-	config_dir = g_strconcat (g_get_home_dir(),"/.config",NULL);
+	config_dir = g_get_user_config_dir();
 	if (g_file_test(config_dir, G_FILE_TEST_EXISTS) == FALSE)
 		g_mkdir_with_parents(config_dir,0700);
 
 	xarchiver_config_dir = g_strconcat(config_dir, "/", PACKAGE, NULL);
-	g_free (config_dir);
 	if (g_file_test(xarchiver_config_dir, G_FILE_TEST_EXISTS) == FALSE)
 		g_mkdir_with_parents(xarchiver_config_dir,0700);
 
 	config_file = g_strconcat(xarchiver_config_dir, "/", PACKAGE, "rc", NULL);
 	g_free (xarchiver_config_dir);
 
+	if (!g_file_test(config_file, G_FILE_TEST_EXISTS))
+	{
+		/* check legacy location of config file */
+		old_dir = g_strconcat(g_get_home_dir(), "/.config/", PACKAGE, NULL);
+		old_config = g_strconcat(old_dir, "/", PACKAGE, "rc", NULL);
+
+		/* move it if it exists */
+		if (g_file_test(old_config, G_FILE_TEST_EXISTS))
+			if (g_rename(old_config, config_file) == 0)
+				g_rmdir(old_dir);
+
+		g_free(old_dir);
+		g_free(old_config);
+	}
+
 	if ( ! g_key_file_load_from_file(xa_key_file,config_file,G_KEY_FILE_KEEP_COMMENTS,NULL) )
 	{
 		/* Write the config file with the default options */
@@ -731,11 +746,6 @@ void xa_prefs_load_options(Prefs_dialog_data *prefs_data)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(add_window->store_path),g_key_file_get_boolean(xa_key_file,PACKAGE,"store_path",NULL));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(add_window->update),g_key_file_get_boolean(xa_key_file,PACKAGE,"updadd",NULL));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(add_window->freshen),g_key_file_get_boolean(xa_key_file,PACKAGE,"freshen",NULL));
-
-		/* set default setting with existing, old config file */
-		if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(add_window->update)) && !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(add_window->freshen)))
-			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(add_window->overwrite), TRUE);
-
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(add_window->recurse),g_key_file_get_boolean(xa_key_file,PACKAGE,"recurse",NULL));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(add_window->solid_archive),g_key_file_get_boolean(xa_key_file,PACKAGE,"solid_archive",NULL));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(add_window->remove_files),g_key_file_get_boolean(xa_key_file,PACKAGE,"remove_files",NULL));
