@@ -32,15 +32,18 @@ static gboolean header_line, data_line, fname_line, last_line;
 
 void xa_rar_check_version (gchar *path)
 {
-	gchar *output, *id;
+	gchar *output = NULL, *id = NULL;
 	gchar version;
 
 	g_spawn_command_line_sync(path, &output, NULL, NULL, NULL);
 
-	id = strstr(output, "\nRAR ");
+	if (output)
+	{
+		id = strstr(output, "\nRAR ");
 
-	if (!id)
-		id = strstr(output, "\nUNRAR ");
+		if (!id)
+			id = strstr(output, "\nUNRAR ");
+	}
 
 	if (id)
 	{
@@ -55,6 +58,8 @@ void xa_rar_check_version (gchar *path)
 
 void xa_rar_ask (XArchive *archive)
 {
+	compressor_t rar_compressor = {TRUE, 1, 3, 5, 1};
+
 	archive->can_test = TRUE;
 	archive->can_extract = TRUE;
 	archive->can_add = archiver[archive->type].is_compressor;
@@ -71,6 +76,9 @@ void xa_rar_ask (XArchive *archive)
 	archive->can_freshen[1] = archiver[archive->type].is_compressor;
 	archive->can_move = archiver[archive->type].is_compressor;
 	archive->can_solid = archiver[archive->type].is_compressor;
+	archive->can_compress = archiver[archive->type].is_compressor;
+	archive->compressor = rar_compressor;
+	archive->compression = archive->compressor.preset;
 }
 
 static gchar *xa_rar_password_str (XArchive *archive)
@@ -556,7 +564,7 @@ gboolean xa_rar_extract (XArchive *archive, GSList *file_list)
 	                      archive->do_touch ? " -tsm-" : "",
 	                      archive->do_overwrite ? " -o+" : (archive->do_update ? " -u" : (archive->do_freshen ? " -f" : " -o-")),
 	                      password_str, " -idp -y ",
-	                      archive->path[1], files->str,
+	                      archive->path[1], " --", files->str,
 	                      " ", archive->extraction_dir, NULL);
 	g_free(password_str);
 	g_string_free(files,TRUE);
@@ -567,14 +575,10 @@ gboolean xa_rar_extract (XArchive *archive, GSList *file_list)
 	return result;
 }
 
-void xa_rar_add (XArchive *archive, GSList *file_list, gchar *compression)
+void xa_rar_add (XArchive *archive, GSList *file_list)
 {
 	GString *files;
-	gchar *password_str, *command, *version_switch;
-
-
-	if (archive->location_path != NULL)
-		archive->child_dir = g_strdup(archive->working_dir);
+	gchar *compression, *password_str, *command, *version_switch;
 
 	if (rar_version >= 5)
 	{
@@ -586,21 +590,21 @@ void xa_rar_add (XArchive *archive, GSList *file_list, gchar *compression)
 	else
 		version_switch = "";
 
-	if (!compression)
-		compression = "3";
+	compression = g_strdup_printf("%hu", archive->compression);
 
 	files = xa_quote_filenames(file_list, NULL, FALSE);
 	password_str = xa_rar_password_str(archive);
 	command = g_strconcat(archiver[archive->type].program[0],
-	                      archive->do_update ? " u" : " a", version_switch,
-	                      archive->do_freshen ? " -f" : "",
+	                      version_switch,
+	                      archive->do_update ? " u" : (archive->do_freshen ? " u -f" : " a"),
 	                      archive->do_move ? " -df" : "",
 	                      archive->do_solid ? " -s" : "",
 	                      " -m", compression,
 	                      password_str, " -idp -y ",
-	                      archive->path[1], files->str, NULL);
+	                      archive->path[1], " --", files->str, NULL);
 	g_free(password_str);
 	g_string_free(files,TRUE);
+	g_free(compression);
 
 	xa_run_command(archive, command);
 	g_free(command);
@@ -612,7 +616,7 @@ void xa_rar_delete (XArchive *archive, GSList *file_list)
 	gchar *command;
 
 	files = xa_quote_filenames(file_list, NULL, FALSE);
-	command = g_strconcat(archiver[archive->type].program[0], " d -idp -y ", archive->path[1], files->str, NULL);
+	command = g_strconcat(archiver[archive->type].program[0], " d -idp -y ", archive->path[1], " --", files->str, NULL);
 	g_string_free(files,TRUE);
 
 	xa_run_command(archive, command);
