@@ -292,7 +292,7 @@ GSList *xa_slist_copy(GSList *list)
 	return g_slist_reverse(y);
 }
 
-void xa_recurse_local_directory (gchar *path, GSList **list, gboolean full_path, gboolean recurse)
+void xa_recurse_local_directory (gchar *path, GSList **list, gboolean full_path, gboolean recurse, gboolean files)
 {
 	DIR *dir;
 	struct dirent *dirlist;
@@ -316,11 +316,11 @@ void xa_recurse_local_directory (gchar *path, GSList **list, gboolean full_path,
 				if (g_file_test(fullname, G_FILE_TEST_IS_DIR))
 				{
 					if (recurse)
-						xa_recurse_local_directory(fullname, list, full_path, recurse);
+						xa_recurse_local_directory(fullname, list, full_path, recurse, files);
 
 					g_free(fullname);
 				}
-				else
+				else if (files)
 					*list = g_slist_prepend(*list, fullname);
 			}
 
@@ -340,9 +340,11 @@ GString *xa_quote_filenames (GSList *file_list, const gchar *escape, gboolean sl
 {
 	GString *files;
 	GSList *list;
+	gboolean hyphen;
 
 	files = g_string_new("");
 	list= file_list;
+	hyphen = (g_strcmp0(escape, "-") == 0);
 
 	while (list)
 	{
@@ -352,13 +354,22 @@ GString *xa_quote_filenames (GSList *file_list, const gchar *escape, gboolean sl
 			xa_remove_slash_from_path(list->data);
 
 		if (escape)
-			escaped = xa_escape_bad_chars(list->data, "\\");
+		{
+			if (hyphen)
+			{
+				if (*(char *) list->data == '-')
+					escaped = g_strdup_printf("./%s", (char *) list->data);
+			}
+			else
+				escaped = xa_escape_bad_chars(list->data, "\\");
+		}
 
 		shellname = g_shell_quote(escaped ? escaped : list->data);
 
 		g_free(escaped);
+		escaped = NULL;
 
-		if (escape)
+		if (escape && !hyphen)
 			escaped = xa_escape_bad_chars(shellname, escape);
 
 		g_string_prepend(files, escaped ? escaped : shellname);
@@ -392,6 +403,32 @@ gchar *xa_quote_shell_command (const gchar *string, gboolean unescaped)
 	return escaped;
 }
 
+GString *xa_quote_dir_contents (const gchar *directory)
+{
+	GDir *dir;
+	const gchar *name;
+	GString *files = g_string_new("");
+
+	dir = g_dir_open(directory, 0, NULL);
+
+	if (dir)
+	{
+		while ((name = g_dir_read_name(dir)))
+		{
+			gchar *quoted = g_shell_quote(name);
+
+			files = g_string_append_c(files, ' ');
+			files = g_string_append(files, quoted);
+
+			g_free(quoted);
+		}
+
+		g_dir_close(dir);
+	}
+
+	return files;
+}
+
 GString *xa_collect_files_in_dir (const gchar *directory)
 {
 	size_t offset;
