@@ -32,6 +32,7 @@
 #include "extract_dialog.h"
 #include "gzip_et_al.h"
 #include "interface.h"
+#include "iso.h"
 #include "main.h"
 #include "mime.h"
 #include "new_dialog.h"
@@ -461,7 +462,7 @@ static void xa_rename_cell_edited (GtkCellRendererText *cell, const gchar *path_
 		archive->do_move = FALSE;
 		archive->do_solid = FALSE;
 
-		xa_execute_add_commands(archive, file_list, FALSE);
+		xa_execute_add_commands(archive, file_list, FALSE, TRUE);
 	}
 done:
 	xa_rename_cell_edited_canceled(GTK_CELL_RENDERER(cell), NULL);
@@ -1813,6 +1814,8 @@ ArchiveType xa_detect_archive_type (const gchar *filename)
 		xa.type = XARCHIVETYPE_ARJ;
 	else if (memcmp(magic, "BZh", 3) == 0)
 		xa.type = XARCHIVETYPE_BZIP2;
+	else if (memcmp(magic, "MSCF\x00\x00\x00\x00", 8) == 0)
+		xa.type = XARCHIVETYPE_CAB;
 	else if (memcmp(magic, "\x1f\x9d", 2) == 0)
 		xa.type = XARCHIVETYPE_COMPRESS;
 	else if (memcmp(magic, "070701", 6) == 0 ||
@@ -1867,6 +1870,13 @@ ArchiveType xa_detect_archive_type (const gchar *filename)
 	}
 	else if (memcmp(magic, "\xed\xab\xee\xdb", 4) == 0)
 		xa.type = XARCHIVETYPE_RPM;
+	else if (memcmp(magic, "sqsh", 4) == 0 || memcmp(magic, "hsqs", 4) == 0)
+	{
+		xa.type = XARCHIVETYPE_SQUASHFS;
+
+		if (g_str_has_suffix(filename, ".snap"))
+			xa.tag = 's';
+	}
 	else if (memcmp(magic, "\xfd" "7zXZ" "\x00", 6) == 0)
 		xa.type = XARCHIVETYPE_XZ;
 	else if (memcmp(magic, "PK" "\x03\x04", 4) == 0 ||
@@ -1890,9 +1900,15 @@ ArchiveType xa_detect_archive_type (const gchar *filename)
 	}
 	else if (g_str_has_suffix(filename, ".exe") && (memcmp(magic, "MZ", 2) == 0))
 		xa = exetype(file);
+	else if (memcmp(magic, "7kSt", 4) == 0 ||
+	         (memcmp(magic, "zPQ", 3) == 0 && magic[3] >= 1))
+		xa.type = XARCHIVETYPE_ZPAQ;
 	else if (memcmp(magic + 1, "\xb5\x2f\xfd", 3) == 0 &&
 	         (*magic == '\x1e' || (*magic >= '\x22' && *magic <= '\x28')))
 		xa.type = XARCHIVETYPE_ZSTD;
+	/* file positioning methods come next to last */
+	else if (isISO(file))
+		xa.type = XARCHIVETYPE_ISO;
 	/* partly heuristic methods must come last */
 	else if (isTar(file))
 		xa.type = XARCHIVETYPE_TAR;
@@ -1941,7 +1957,6 @@ void xa_create_liststore (XArchive *archive, const gchar *titles[])
 	gtk_tree_view_column_set_resizable (column,TRUE);
 	gtk_tree_view_column_set_sort_column_id (column,1);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (archive->treeview),column);
-	gtk_tree_view_column_set_sizing (column,GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	g_signal_connect(archive->text_renderer, "editing-canceled", G_CALLBACK(xa_rename_cell_edited_canceled), archive);
 	g_signal_connect(archive->text_renderer, "edited", G_CALLBACK(xa_rename_cell_edited), archive);
 
@@ -2362,7 +2377,7 @@ failed:
 		archive[idx]->do_move = FALSE;
 		archive[idx]->do_solid = FALSE;
 
-		xa_execute_add_commands(archive[idx], list, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->allow_sub_dir)));
+		xa_execute_add_commands(archive[idx], list, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->allow_sub_dir)), TRUE);
 	}
 	else
 	{
@@ -2597,6 +2612,7 @@ void xa_location_entry_activated (GtkEntry *entry,gpointer user_data)
 
 gboolean xa_treeview_mouse_button_press (GtkWidget *widget, GdkEventButton *event, XArchive *archive)
 {
+	static guint32 last_button1;
 	XEntry *entry;
 	GtkTreePath *path;
 	GtkTreeIter iter;
@@ -2620,14 +2636,17 @@ gboolean xa_treeview_mouse_button_press (GtkWidget *widget, GdkEventButton *even
 
 		if (event->button == 1)
 		{
-			GdkModifierType state;
+			if (event->time - last_button1 > 250)
+			{
+				last_button1 = event->time;
 
-			gdk_event_get_state((GdkEvent *) event, &state);
-
-			if (gtk_tree_selection_iter_is_selected(selection, &iter) && ((state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK)) == 0))
-				return TRUE;
+				if (gtk_tree_selection_iter_is_selected(selection, &iter) && ((event->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK)) == 0))
+					return TRUE;
+				else
+					return FALSE;
+			}
 			else
-				return FALSE;
+				event->button = 2;
 		}
 
 		if (!gtk_tree_selection_iter_is_selected(selection, &iter))
@@ -2764,7 +2783,7 @@ void xa_clipboard_paste (GtkMenuItem *item, gpointer user_data)
 	archive[idx]->do_move = FALSE;
 	archive[idx]->do_solid = FALSE;
 
-	xa_execute_add_commands(archive[idx], XA_Clipboard.paths, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->allow_sub_dir)));
+	xa_execute_add_commands(archive[idx], XA_Clipboard.paths, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->allow_sub_dir)), TRUE);
 
 	if (archive[idx]->status == XARCHIVESTATUS_ERROR)
 		return;
